# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""

import zipfile
import numpy
import re
import collections
from pyphant.core import (Worker, Connectors, DataContainer)
from pyphant.quantities import Quantity, isQuantity
import pkg_resources


def makeRoot(pixelName, ll):
    return {}


def startSection(root, dataSections, l):
    if l == u'MESSDATEN':
        root[l] = []
    elif l == u'SPALTENBESCHRIFTUNG':
        root[l] = []
    else:
        root[l] = {}
        dataSections.append(l)
    return root[l], l


def addAttributeLine(section, l):
    ff = l.split('=')
    section[ff[0]] = ff[1]


def addDataColumn(section, l):
    ff = l.split('=')
    section.append(ff[1])


def addTableLine(section, l):
    ff = l.split()
    section.append([float(f) for f in ff])


LINE_HANDLER = collections.defaultdict(lambda: addAttributeLine)
LINE_HANDLER[u'Section'] = startSection
LINE_HANDLER[u'SPALTENBESCHRIFTUNG'] = addDataColumn
LINE_HANDLER[u'MESSDATEN'] = addTableLine


def readZipFile(filename, data=None, subscriber=0):
    if data == None:
        data = []
    z = zipfile.ZipFile(filename, 'r')
    names = z.namelist()
    total = len(names)
    for i, pixelName in enumerate(names):
        b = z.read(pixelName)
        root, dataSections = readSingleFile(b, pixelName)
        data.append(root)
        subscriber %= float(i) / total * 100.0
    return data, dataSections


def readDataFile(filename, data=None):
    if data == None:
        data = []
    filehandle = open(filename, 'r')
    dat = filehandle.read()
    filehandle.close()
    root, dataSections = readSingleFile(dat, filename)
    data.append(root)
    return data, dataSections


def createFieldContainer(key, array):
    def longname2unit(name):
        reg = re.compile(r"\[([^]]*)\]")
        m = reg.search(name)
        if not m or m.group(1) == "counts" or m.group(1) == "":
            return 1.0
        else:
            return Quantity('1 ' + str(m.group(1)))
    fieldUnit = longname2unit(key)
    if isQuantity(fieldUnit):
        fieldContainer = DataContainer.FieldContainer(
            numpy.array(array), unit=fieldUnit, longname=key
            )
    else:
        try:
            quantities = [
                Quantity(string.encode('latin-1')) for string in array
                ]
            firstUnit = quantities[0].unit
            scaledArray = [
                quant.inUnitsOf(firstUnit).value for quant in quantities
                ]
            fieldContainer = DataContainer.FieldContainer(
                numpy.array(scaledArray),
                unit='1. %s' % firstUnit.name(),
                longname=key
                )
        except:
            fieldContainer = DataContainer.FieldContainer(
                numpy.array(array),
                unit=fieldUnit,
                longname=key
                )
    return fieldContainer


def loadOscFromFile(filename, subscriber=0):
    try:
        data, dataSections = readZipFile(filename, subscriber=subscriber)
    except zipfile.BadZipfile:
        data, dataSections = readDataFile(filename)
    container = constructTemplate(data, dataSections)
    for d in data:
        for dicname in dataSections:
            for k in d[dicname].keys():
                container[k].append(d[dicname][k])
        for i, col in enumerate(d[u'SPALTENBESCHRIFTUNG']):
            container[col].append(d[u'MESSDATEN'][:, i])
    cols = [createFieldContainer(k, v) for k, v in container.iteritems()]
    if container.has_key('KOMMENTAR'):
        title = container[u'KOMMENTAR'][0]
    else:
        title = ''
    return DataContainer.SampleContainer(cols, longname=title)


def constructTemplate(data, dataSections):
    template = data[0]
    container = {}
    for k in reduce(
        lambda x, y: x + y,
        [template[secName].keys() for secName in dataSections] + \
            [template[u'SPALTENBESCHRIFTUNG']],
        []
        ):
        container[k] = []
    return container


def readSingleFile(b, pixelName):
    d = unicode(b, 'cp1252')
    ll = d.splitlines()
    root = makeRoot(pixelName, ll)
    dataSections = []
    section = root
    secId = 'root'
    for l in ll:
        if l.startswith('[') and l.endswith(']'):
            section, secId = LINE_HANDLER[u'Section'](
                root, dataSections, l[1:-1]
                )
        else:
            LINE_HANDLER[secId](section, l)
    root[u'MESSDATEN'] = numpy.array(root[u'MESSDATEN'])
    return (root, dataSections)


class OscLoader(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Data Loader"
    _params = [("filename", u"Filename", "", Connectors.SUBTYPE_FILE)]

    @Worker.plug(Connectors.TYPE_ARRAY)
    def loadOsc(self, subscriber=0):
        result = loadOscFromFile(self.paramFilename.value, subscriber)
        result.seal()
        return result



# -*- coding: utf-8 -*-

# Copyright (c) 2008, Rectorate of the University of Freiburg
# Copyright (c) 2009-2011, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""Provides unittest classes TestMRA and TestMRADiscontinuousDiscretisation.
"""



import sys,copy
import unittest
import numpy, scipy, scipy.optimize
import OSC.MRA as MRA
from pyphant.core import DataContainer as DC
from pyphant.tests.Helpers import fixedPoints


class TestMRA(unittest.TestCase):
    """Sets up a mirror symmetric bistable potential with a continuous
    distretisation and computes its local extrema and the respective
    curvatures."""
    def setUp(self):
        self.n = 1000
        self.u = numpy.linspace(-1.5, 1.5, self.n)
        self.LAMBDA = 0.5
        self.kappa1 = 0.0
        self.xField = DC.FieldContainer(
            self.u,
            unit = '1 m',
            longname = 'position',
            shortname = 'x'
            )
        self.V = DC.FieldContainer(
            -self.LAMBDA / 2 * self.u ** 2 + self.u ** 4 / 4 \
            - self.u * self.kappa1,
            unit =' 1 V', dimensions = [self.xField],
            longname = 'electric potential',
            shortname = r'\varphi'
            )

    def testMinima(self):
        """
        Test the correct computation of all local minima
        for a bistable potential.
        """
        #Predict result
        x0, curv, mask = fixedPoints(
            numpy.array([self.LAMBDA]), kappa1=self.kappa1
            )
        expectedResult = DC.FieldContainer(
            numpy.extract(curv[0]>0,x0[0]),
            unit = self.xField.unit,
            longname = 'position of the local minima of electric potential',
            shortname = 'x_0'
            )
        #Retrieve result from worker
        w = MRA.MRA(None)
        w.paramScale.value = "1.0m"
        result = w.mra(self.V)
        #Testing
        i = numpy.array(range(self.n))
        index = numpy.logical_and(self.u > 0.7, self.u < 0.72)
        index = MRA.findMinima(self.V.data, 5)
        numpy.testing.assert_array_almost_equal(
            result[r'x_{min}'].data, expectedResult.data, 4
            )

    def testMaxima(self):
        """
        Test the correct computation of all local maxima
        for a bistable potential.
        """
        #Predict result
        x0, curv, mask = fixedPoints(
            numpy.array([self.LAMBDA]), kappa1=self.kappa1
            )
        expectedResult = DC.FieldContainer(
            numpy.extract(curv[0] > 0, x0[0]),
            unit = self.xField.unit,
            longname = 'position of the local minima of electric potential',
            shortname = 'x_0')
        #Retrieve result from worker
        w = MRA.MRA(None)
        w.paramScale.value = "1.0m"
        V = copy.deepcopy(self.V)
        V.data*=-1
        result = w.mra(V)
        #Testing
        numpy.testing.assert_array_almost_equal(
            result[r'x_{max}'].data,expectedResult.data, 4
            )


class TestExtremumFinderTable(unittest.TestCase):
    """Sets up a mirror symmetric bistable potential with a continuous
    distretisation and computes its local extrema and the respective
    curvatures."""
    def setUp(self):
        self.n = 1000
        self.m = 10
        self.kappa1=0.0
        self.errLevelPos = 2
        self.errLevelCurv= 5
        self.test = DC.assertEqual

    def prepareDimensions(self):
        X,LAMB = scipy.meshgrid(numpy.linspace(-1.5,1.5,self.n),
                                numpy.linspace(-1.0,1.0,self.m))
        self.lambDim = LAMB[:,0]
        self.xDim = X

    def testRoots(self):
        """
        Test the correct computation of all local extrema
        for a bistable potential.
        """
        #Prepare dimensions
        self.prepareDimensions()
        lambField = DC.FieldContainer(
            self.lambDim,
            unit='1 V / m**3',
            longname='parameter',
            shortname='\lambda'
            )
        xField = DC.FieldContainer(
            self.xDim[0],
            unit='1 m',
            longname='position',
            shortname='x'
            )
        #Prepare potential
        V = []
        for i in xrange(len(lambField.data)):
            u = xField.data
            V.append(
                -lambField.data[i] / 2 * u ** 2 + u ** 4 / 4 - u * self.kappa1
                )
        self.V = DC.FieldContainer(
            numpy.array(V), unit='1 V', dimensions=[lambField, xField],
            longname = 'electric potential',
            shortname = r'\varphi'
            )
        #Predict result
        x0, curv, mask = fixedPoints(lambField.data, kappa1=self.kappa1)
        x0 = numpy.where(curv > 0, x0, numpy.NaN)
        data = x0[:, ::2]
        dims = [DC.generateIndex(0, 2), lambField]
        expectedResult = DC.FieldContainer(
            data.transpose(),
            unit=xField.unit,
            mask=numpy.isnan(data).transpose(),
            dimensions=dims,
            longname='position of the local extrema of electric potential',
            shortname='x_0'
            )
        #Configure worker
        w = MRA.MRA(None)
        w.paramScale.value = "1.0m"
        #Retrieve result from worker
        result = copy.deepcopy(w.mra(self.V))['x_{min}']
        result.error=None
        self.test(result,expectedResult,1e-2,1e-2)


class TestFindExtrema(unittest.TestCase):
    """
    Sets up a mirror symmetric function with one local extremum
    and compares the result with argmin/argmax.
    """
    def setUp(self):
        self.a = numpy.abs(numpy.linspace(-1, 1, 11))
        self.p = numpy.zeros(10)
        self.p[3:5] = -1

    def testFindMinimum(self):
        """Test the correct computation of a local minimum."""
        self.assertEqual(numpy.argmin(self.a), MRA.findMinima(self.a, 5)[0])

    def testPlateau(self):
        """Test the correct computation of a local minimum."""
        self.assertEqual(numpy.array([3]), MRA.findMinima(self.p, 5)[0])

    def testFindMaximum(self):
        """Test the correct computation of a local minimum."""
        self.assertEqual(numpy.argmax(-self.a), MRA.findMaxima(-self.a, 5)[0])

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

# Copyright (c) 2007-2008, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
The Smoother Worker is a class of Pyphant's OSC Toolbox. It is used to
reduce noise from a field. The numer of smoothing runs can be edited.
"""

import numpy
from pyphant.core import (Worker, Connectors, DataContainer)
import scipy.interpolate
import logging
import copy
import pkg_resources
_logger = logging.getLogger("pyphant")


class Smoother(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Smoother"
    _sockets = [("osc", Connectors.TYPE_IMAGE)]
    _params = [
        (
            "smoothing",
            "Fit smoothing (0 no smoothing, larger value more smoothing)",
            "1",
            None
            )
        ]

    @Worker.plug(Connectors.TYPE_IMAGE)
    def smooth(self, osc, subscriber=0):
        s = float(self.paramSmoothing.value)
        xDim = osc.dimensions[-1]
        x = xDim.data
        smoothedData = []
        count = osc.data.shape[0]
        for i in xrange(count):
            y = osc.data[i]
            if osc.error != None:
                error = copy.deepcopy(osc.error[i])
                error[error == 0] = numpy.finfo(type(0.3)).eps
                weights = 1.0 / (error ** 2)
                smoothYTCK, fp, ier, msg = scipy.interpolate.splrep(
                    x, y, w=weights, s=s,
                    full_output=True, task=0
                    )
            else:
                smoothYTCK, fp, ier, msg = scipy.interpolate.splrep(
                    x, y, s=s,
                    full_output=True, task=0
                    )
            if ier > 0:
                _logger.warning(
                    "There was a problem in fitting: %i: %s, fp: %f" % (
                        ier, msg, fp
                        )
                    )
            smoothY = scipy.interpolate.splev(x, smoothYTCK)
            smoothedData.append(smoothY)
            subscriber %= float(i) * 100.0 / float(count)
        sya = numpy.array(smoothedData)
        result = DataContainer.FieldContainer(
            sya,
            error=osc.error,
            unit=osc.unit,
            dimensions=osc.dimensions,
            longname=u'Smoothed %s' % osc.longname,
            shortname='\\widetilde{%s}' % osc.shortname
            )
        result.seal()
        return result



# -*- coding: utf-8 -*-

# Copyright (c) 2007-2009, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""

import numpy
from pyphant.core import (Worker, Connectors, DataContainer)
from pyphant.quantities import Quantity
from OSC.OscAbsorption import grid2Index
import logging
import copy
import pkg_resources


class OscMapper(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Mapper"
    _sockets = [("osc", Connectors.TYPE_ARRAY)]
    _params = [("xAxis", u"x-Axis", [u"horizontal_table_position"], None),
               ("yAxis", u"y-Axis", [u"vertical_table_position"], None),
               ("field", u"Field", [u"thickness"], None),
               ("extentX", u"Extension of x-axis [%%]", 10, None),
               ("extentY", u"Extension of y-axis [%%]", 10, None),
               ("overrideV", u"Override value limits", False, None),
               ("vmin", u"Minimal value", "0 nm", None),
               ("vmax", u"Maximal value", "100 nm", None)]

    def inithook(self):
        self._logger = logging.getLogger("pyphant")

    def refreshParams(self, subscriber=None):
        if self.socketOsc.isFull():
            templ = self.socketOsc.getResult(subscriber)
            colNames = templ.longnames.keys()
            self.paramXAxis.possibleValues = colNames
            self.paramYAxis.possibleValues = colNames
            self.paramField.possibleValues = colNames

    def calcNormal(self, osc, xCon, yCon, fCon, xf, yf, h):
        xOff, xStep, xInd = grid2Index(xf, self.paramExtentX.value)
        yOff, yStep, yInd = grid2Index(yf, self.paramExtentY.value)
        xMax = xInd.maxV # never used!
        yMax = yInd.maxV # never used!
        xDim = DataContainer.FieldContainer(
            numpy.linspace(xInd.minV, xInd.maxV, xInd.stepCount) - 0.5 * xStep,
            xCon.unit,
            longname=xCon.longname,
            shortname=xCon.shortname
            )
        yDim = DataContainer.FieldContainer(
            numpy.linspace(yInd.minV, yInd.maxV, yInd.stepCount) - 0.5 * yStep,
            yCon.unit,
            longname=yCon.longname,
            shortname=yCon.shortname
            )
        img = numpy.ones((yInd.stepCount, xInd.stepCount),
                         dtype='float') * numpy.NaN
        mask = numpy.ones((yInd.stepCount, xInd.stepCount), dtype='bool')
        for i in xrange(xf.size):
            xi = xInd[xf[i]]
            yi = yInd[yf[i]]
            if not mask[yi, xi]:
                self._logger.warning(
                    "Duplicate data for pixel (%.4g,%.4g). "
                    "Using first found value. "
                    "Is your data corrupt?" % (xf[i], yf[i])
                    )
            else:
                img[yi, xi] = h[i]
                if h[i] > 0:
                    mask[yi, xi] = False
        result = DataContainer.FieldContainer(
            img, fCon.unit, mask=mask,
            dimensions=[yDim, xDim],
            longname=u'Map of %s' % fCon.longname,
            shortname=fCon.shortname
            )
        return result

    @Worker.plug(Connectors.TYPE_IMAGE)
    def mapHeights(self, osc, subscriber=0):
        xCon = osc[self.paramXAxis.value]
        yCon = osc[self.paramYAxis.value]
        fCon = osc[self.paramField.value]
        cons = [copy.deepcopy(data) for data in [xCon, yCon, fCon]]
        xCon, yCon, fCon = cons
        for con in cons:
            con.longname = con.longname.replace('_', ' ')
            con.data = con.data.astype('float')
        xf, yf, h = tuple([con.data for con in cons])
        result = self.calcNormal(osc, xCon, yCon, fCon, xf, yf, h)
        if self.paramOverrideV.value:
            vs = str(self.paramVmin.value), str(self.paramVmax.value)
            try:
                vs = [Quantity(v) for v in vs]
            except SyntaxError:
                vs = [float(v) for v in vs]
            result.attributes['vmin'] = vs[0]
            result.attributes['vmax'] = vs[1]
        result.seal()
        return result



# -*- coding: utf-8 -*-

# Copyright (c) 2007-2009, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""

import numpy
from pyphant.core import (Worker, Connectors, DataContainer)
from pyphant.quantities import Quantity
import logging
import pkg_resources


class IndexDict(object):
    def __init__(self, minV, maxV, step):
        self.minV = minV
        self.maxV = maxV
        self.step = step

    def __getitem__(self, key):
        return round((key - self.minV) / self.step)

    stepCount = property(lambda self:
                         round((self.maxV - self.minV) / self.step))


def grid2Index(field, extension=0):
    _logger = logging.getLogger("pyphant")
    f = numpy.unique(field)
    fmax = f.max()
    fmin = f.min()
    amplitude = fmax - fmin
    extent = extension / 200. * amplitude
    f.sort()
    fd = f[1:] - f[:-1]
    try:
        fstep = fd.min()
        offset = fmin - extent
        fmax += extent
    except ValueError:  # Occurs if all elements of array are equal
        fstep = f[0]
        offset = f[0] - extent
        fmax = f[0] + extent
    fic = (f - fmin) / fstep
    fi = fic.round().astype('int')
    if numpy.allclose(fi, fic):
        indexDict = IndexDict(offset, fmax, fstep)
    else:
        _logger.warning(u"There seems to be a problem with the discretisation.\
                            We are choosing 100 points.")
        fstep = (fmax - offset) / 100.
        indexDict = IndexDict(offset, fmax, fstep)
    return (offset, fstep, indexDict)


def removePeak(Abso, lower, upper):
    dim = Abso.dimensions[-1]
    minVal = lower / dim.unit
    maxVal = upper / dim.unit
    high_part = numpy.argwhere(minVal < dim.data).flatten()
    low_part = numpy.argwhere(dim.data < maxVal).flatten()
    lamp_interval = numpy.intersect1d(high_part, low_part)
    min_index = lamp_interval[0]
    max_index = lamp_interval[-1]
    steps = max_index - min_index
    for row in Abso.data:
        min_value = row[min_index]
        max_value = row[max_index]
        step_size = (max_value - min_value) / steps
        row[lamp_interval] = min_value + (lamp_interval - min_index) *\
                                    step_size


class OscAbsorptionCalculator(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Compute Absorption"
    _sockets = [("osc", Connectors.TYPE_ARRAY)]
    _params = [("clipping", "Clipping", 1, None),
               ("mask_lamp", "Mask lamp", 0, None)]

    def inithook(self):
        self._logger = logging.getLogger("pyphant")

    @Worker.plug(Connectors.TYPE_IMAGE)
    def calcAbsorption(self, osc, subscriber=0):
        I = osc[u'I']
        I_d = osc[u'I_d']
        I_0 = osc[u'I_0']
        A = - ((I.data - I_d.data) / (I_0.data - I_d.data) - 1)
        if self.paramClipping.value == 1:
            A[A > 1] = 1
            A[A < 0] = 0
        Abso = DataContainer.FieldContainer(A,
                                            longname=u'absorption',
                                            shortname=ur'\tilde{A}')
        Abso.dimensions[-1] = I.dimensions[-1]
        if self.paramMask_lamp.value == 1:
            removePeak(Abso, Quantity('654nm'), Quantity('660nm'))
            removePeak(Abso, Quantity('920nm'), Quantity('980nm'))
        Abso.seal()
        return Abso



# -*- coding: utf-8 -*-

# Copyright (c) 2008-2009, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""

import numpy
from pyphant.core import (Worker, Connectors, DataContainer)
import pkg_resources


class EstimateParameterFromValues(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Estimate Parameter from Values"
    _sockets = [
        ("model", Connectors.TYPE_ARRAY),
        ("experimental", Connectors.TYPE_ARRAY)
        ]
    _params = [
        ("minima_model", u"Minima in the model", [u"Minima"], None),
        ("maxima_model", u"Maxima in the model", [u"Maxima"], None),
        (
            "minima_experimental",
            u"Minima in the experiment",
            [u"Minima"],
            None
            ),
        (
            "maxima_experimental",
            u"Maxima in the experiment",
            [u"Maxima"],
            None
            ),
        ("extentX", u"Extension of x-axis [%%]", 10, None),
        ("extentY", u"Extension of y-axis [%%]", 10, None)
        ]

    def refreshParams(self, subscriber=None):
        if self.socketModel.isFull():
            templ = self.socketModel.getResult(subscriber)
            self.paramMinima_model.possibleValues = templ.longnames.keys()
            self.paramMaxima_model.possibleValues = templ.longnames.keys()
        if self.socketExperimental.isFull():
            templ = self.socketExperimental.getResult(subscriber)
            self.paramMinima_experimental.possibleValues = templ.longnames.keys()
            self.paramMaxima_experimental.possibleValues = templ.longnames.keys()

    @Worker.plug(Connectors.TYPE_IMAGE)
    def compute(self, model, experimental, subscriber=1):
        minima_model = model[self.paramMinima_model.value]
        maxima_model = model[self.paramMaxima_model.value]
        minima_experimental = experimental[self.paramMinima_experimental.value]
        minima_experimental = minima_experimental.inUnitsOf(minima_model)
        minima = minima_experimental.data.transpose()
        if minima_experimental.error != None:
            minima_error = iter(minima_experimental.error.transpose())
        else:
            minima_error = None
        maxima_experimental = experimental[self.paramMaxima_experimental.value]
        maxima_experimental = maxima_experimental.inUnitsOf(maxima_model)
        maxima = maxima_experimental.data.transpose()
        if maxima_experimental.error != None:
            maxima_error = iter(maxima_experimental.error.transpose())
        else:
            maxima_error = None
        parameter = []
        inc = 100.0 / float(len(minima))
        acc = inc
        subscriber %= acc
        mask = []
        for row_minima, row_maxima in zip(minima, maxima):
            if minima_error:
                filtered_minima_error = filter(
                    lambda c: not numpy.isnan(c), minima_error.next())
            else:
                filtered_minima_error = None
            if maxima_error:
                filtered_maxima_error = filter(
                    lambda c: not numpy.isnan(c), maxima_error.next())
            else:
                filtered_maxima_error = None
            row_minima = numpy.array(
                filter(lambda c: not numpy.isnan(c), row_minima)
                )
            row_maxima = numpy.array(
                filter(lambda c: not numpy.isnan(c), row_minima)
                )
            grades = []
            for rm_minima, rm_maxima in zip(
                minima_model.data.transpose(), maxima_model.data.transpose()
                ):
                rm_minima = filter(lambda c: not numpy.isnan(c), rm_minima)
                rm_maxima = filter(lambda c: not numpy.isnan(c), rm_maxima)
                grade = 0
                if len(rm_minima) == len(row_minima):
                    grade = sum(numpy.abs(numpy.array(rm_minima) - row_minima))
                if len(rm_maxima) == len(row_maxima):
                    grade += sum(
                        numpy.abs(numpy.array(rm_maxima) - row_maxima)
                        )
                if grade == 0:
                    grades.append(numpy.nan)
                    continue
                grades.append(grade)
            grades = numpy.array(grades)
            i = numpy.nanargmin(grades)
            if numpy.isnan(i):
                mask.append(True)
                parameter.append(numpy.nan)
            else:
                mask.append(False)
                parameter.append(minima_model.dimensions[1].data[i])
            acc += inc
            subscriber %= acc
        result = DataContainer.FieldContainer(
            numpy.array(parameter),
            mask=numpy.array(mask),
            longname=minima_model.dimensions[-1].longname,
            shortname=minima_model.dimensions[-1].shortname,
            unit=minima_model.dimensions[-1].unit
            )
        result.seal()
        return result



# -*- coding: utf-8 -*-

# Copyright (c) 2008, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""

import numpy
from pyphant.core import (Worker, Connectors, DataContainer)
import scipy.interpolate
import copy
import pkg_resources


class ThicknessModeller(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Coat Thickness Model"
    _sockets = [("osc", Connectors.TYPE_ARRAY)]

    @Worker.plug(Connectors.TYPE_IMAGE)
    def calcAbsorption(self, osc, subscriber=0):
        A = copy.deepcopy(osc[u'absorption'])
        heights = osc[u'thickness']
        indexMap = dict([(h, i) for i, h in enumerate(heights.data)])
        h = copy.deepcopy(heights.data)
        h.sort()
        data = numpy.vstack([copy.deepcopy(A.data[indexMap[i]]) for i in h ])
        height = copy.deepcopy(heights)
        height.data = h
        attr = copy.deepcopy(A.attributes).update(osc.attributes)
        result = DataContainer.FieldContainer(
            data, unit=A.unit,
            longname=A.longname,
            shortname=A.shortname,
            attributes=attr,
            dimensions=[height, A.dimensions[-1]]
            )
        result.seal()
        return result


class ThicknessSmoother(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Coat Thickness Smoother"
    _sockets = [("osc", Connectors.TYPE_IMAGE)]

    @Worker.plug(Connectors.TYPE_IMAGE)
    def calcSmoother(self, osc, subscriber=0):
        x, y = (
            copy.deepcopy(osc.dimensions[0].data),
            copy.deepcopy(osc.dimensions[1].data)
            )
        X, Y = scipy.meshgrid(x, y)
        Z = osc.data  # scipy.diff(osc.data,2,0)
        spline = scipy.interpolate.interp2d(
            X.flatten(), Y.flatten(), Z.flatten()
            )
        X, Y = scipy.meshgrid(
            numpy.linspace(x.min(), x.max(), 10),
            numpy.linspace(y.min(), y.max(), 10)
            )
        Z = spline(X.flatten(), Y.flatten())
        Z.resize((10, 10))
        result = DataContainer.FieldContainer(Z, dimensions=osc.dimensions)
        result.seal()
        return result



# -*- coding: utf-8 -*-

# Copyright (c) 2008, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""Provides unittest class TestErrorEstimation.
"""



import sys
import unittest
import numpy
import OSC.ErrorEstimator as OE

class TestLocalNoise(unittest.TestCase):
    """Tests the estimation of local noise on basis of sample standard deviations."""
    def setUp(self):
        self.data = numpy.arange(10).astype('float')

    def testLinearOddSamples(self):
        """Tests the correct computation of sample standard deviation for a straight line with an odd number of samples."""
        expected = numpy.resize(1.0,self.data.shape)
        stdRand = numpy.sqrt((2.0*(1.0-2.0/3)**2+4.0/9.0)*0.5)
        expected[0] = stdRand
        expected[-1]= stdRand
        result = OE.localNoise(self.data,samples=3)

        numpy.testing.assert_array_almost_equal(expected,result,err_msg='Expected local noise is %s, but should be %s.' % (result,expected))

    def testLinearEvenSamples(self):
        """Tests the correct computation of sample standard deviation for a straight line with an even set of samples.
        """
        std = numpy.sqrt(5./3)
        expected = numpy.resize(std,self.data.shape)
        stdRand = numpy.sqrt(2./3)
        expected[:2] = stdRand
        expected[-1:]= stdRand
        result = OE.localNoise(self.data,samples=4)

        numpy.testing.assert_array_almost_equal(expected,result,err_msg='Expected local noise is %s, but should be %s.' % (result,expected))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

# Copyright (c) 2008-2010, Rectorate of the University of Freiburg
# Copyright (c) 2009-2011  Andreas W. Liehr (liehr@users.sourceforge.net)
# Copyright (c) 2010, Klaus Zimmermann (zklaus@users.sourceforge.net)
# all rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""

import numpy
from pyphant.core import (Worker, Connectors, DataContainer)
import scipy.interpolate
from pyphant import quantities
from scipy.special import ive
from scipy.signal import convolve
import pkg_resources


def findMaxima(fieldData, numb_edge, lastExtrema=None):
    return findMinima(-fieldData, numb_edge, lastExtrema)


def findMinima(fieldData, numb_edge, lastExtrema=None):
    leftGreater = fieldData[:-2] > fieldData[1:-1]
    leftEqual = fieldData[:-2] == fieldData[1:-1]
    rightGreater = fieldData[1:-1] < fieldData[2:]
    rightEqual = fieldData[2:] == fieldData[1:-1]
    minima_c = numpy.logical_and(leftGreater, rightGreater)  # Minima
    minima_le = numpy.logical_and(leftGreater, rightEqual)
    minima_re = numpy.logical_and(rightGreater, leftEqual)
    minima_e = numpy.logical_and(minima_le[:-1], minima_re[1:])
    minima = 1 + numpy.logical_or(minima_c[:-1], minima_e).nonzero()[0]
    edge = len(fieldData) / numb_edge
    minima = minima[
        numpy.logical_or(edge < minima, minima < (len(fieldData) - edge))
        ]
    if lastExtrema == None or len(minima) == 0 or \
            len(lastExtrema) == len(minima):
        return minima
    trackedMinima = []
    for lastMinimum in lastExtrema:
        distance = (minima - lastMinimum) ** 2
        trackedMinima.append(distance.argmin())
    result = minima[trackedMinima]
    return result


def convolveMRA(field, sigma):
    if sigma == 0:
        return field.data
    n = int(len(field.dimensions[-1].data) / 2)
    kernel = ive(numpy.arange(-n, n), sigma)
    return convolve(field.data, kernel, mode='same')


class MraError(RuntimeError):
    def __init__(self, msg, convolvedField):
        RuntimeError.__init__(self, msg)
        self.convolvedField = convolvedField


def mra1d(dim, field, n, numb_edge):
    sigmaSpace = numpy.linspace(n, 0, 10)
    convolvedField = convolveMRA(field, sigmaSpace[0])
    firstMinima = lastMinima = findMinima(convolvedField, numb_edge, None)
    firstMaxima = lastMaxima = findMaxima(convolvedField, numb_edge, None)
    if len(firstMinima) == 0 and len(firstMaxima) == 0:
        raise MraError("No Extrema found at sigma level %s" % sigmaSpace[0],
                       convolvedField)
    for sigma in sigmaSpace[1:]:
        convolvedField = convolveMRA(field, sigma)
        lastMinima = findMinima(convolvedField, numb_edge, lastMinima)
        lastMaxima = findMaxima(convolvedField, numb_edge, lastMaxima)
    if len(lastMinima) > 0 and len(firstMinima) > 0:
        pos_minima = dim.data[numpy.array(lastMinima)]
        error_minima = numpy.abs(
            pos_minima - dim.data[numpy.array(firstMinima)]
            )
    else:
        pos_minima = numpy.array([], dtype=dim.data.dtype)
        error_minima = numpy.array([], dtype=dim.data.dtype)
    if len(lastMaxima) > 0 and len(firstMaxima) > 0:
        pos_maxima = dim.data[numpy.array(lastMaxima)]
        error_maxima = numpy.abs(
            pos_maxima - dim.data[numpy.array(firstMaxima)]
            )
    else:
        pos_maxima = numpy.array([], dtype=dim.data.dtype)
        error_maxima = numpy.array([], dtype=dim.data.dtype)
    result = ((pos_minima, error_minima), (pos_maxima, error_maxima))
    return result


def pos_error_to_data_container(p_e):
    n = max(map(lambda (p, e): len(p), p_e))
    m = len(p_e)
    pos = numpy.ones((m, n), 'float') * numpy.NaN
    error = pos.copy()
    for i in xrange(m):
        for j in xrange(len(p_e[i][0])):
            pos[i, j] = p_e[i][0][j]
            error[i, j] = p_e[i][1][j]
    return n, pos, error


class MRA(Worker.Worker):
    API = 2
    VERSION = 2
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Multi Resolution Analyzer"
    _sockets = [("field", Connectors.TYPE_IMAGE)]
    _params = [
        ("scale", u"Scale", "200 nm", None),
        ("numb_edge", u"Width of edge to discard extrema in [%%]", 5, None),
        ("longname", u"Name of result", 'default', None),
        ("symbol", u"Symbol of result", 'default', None)
        ]

    @Worker.plug(Connectors.TYPE_ARRAY)
    def mra(self, field, subscriber=0):
        dim = field.dimensions[-1]
        try:
            scale = quantities.Quantity(self.paramScale.value.encode('utf-8'))
        except:
            scale = float(self.paramScale.value)
        numb_edge = 100.0 / self.paramNumb_edge.value
        d = scipy.diff(dim.data)
        numpy.testing.assert_array_almost_equal(d.min(), d.max(), 4)
        sigmaMax = scale / (d[0] * dim.unit)
        if len(field.data.shape) > 1:
            p_e = []
            inc = 100. / len(field.data)
            acc = 0.
            for field1d in field:
                try:
                    p_e.append(mra1d(dim, field1d, sigmaMax, numb_edge))
                except MraError:
                    p_e.append((([], []), ([], [])))
                acc += inc
                subscriber %= acc
            minima, maxima = zip(*p_e)
            n_min, pos_min, err_min = pos_error_to_data_container(minima)
            n_max, pos_max, err_max = pos_error_to_data_container(maxima)
            dims_min = [
                DataContainer.generateIndex(0, n_min),
                field.dimensions[0]
                ]
            dims_max = [
                DataContainer.generateIndex(0, n_max),
                field.dimensions[0]
                ]
        else:
            (pos_min, err_min), (pos_max, err_max) = mra1d(
                dim, field, sigmaMax, numb_edge
                )
            dims_min = [DataContainer.generateIndex(0, len(pos_min))]
            dims_max = [DataContainer.generateIndex(0, len(pos_max))]
            subscriber %= 100.
        minima = DataContainer.FieldContainer(
            pos_min.transpose(),
            error=err_min.transpose(),
            unit=dim.unit,
            dimensions=dims_min,
            mask=numpy.isnan(pos_min).transpose(),
            longname="%s of the local %s of %s" % (
                dim.longname, "minima", field.longname
                ),
            shortname="%s_{min}" % dim.shortname
            )
        maxima = DataContainer.FieldContainer(
            pos_max.transpose(),
            error=err_max.transpose(),
            unit=dim.unit,
            dimensions=dims_max,
            mask=numpy.isnan(pos_max).transpose(),
            longname="%s of the local %s of %s" % (
                dim.longname, "maxima", field.longname
                ),
            shortname="%s_{max}" % dim.shortname
            )
        roots = DataContainer.SampleContainer(
            [minima, maxima],
            longname="%s of the local %s of %s" % (
                dim.longname, "extrema", field.longname
                ),
            shortname="%s_{extrem}" % dim.shortname
            )
        if self.paramLongname.value != 'default':
            roots.longname = self.paramLongname.value
        if self.paramSymbol.value != 'default':
            roots.shortname = self.paramSymbol.value
        roots.seal()
        return roots


def main():
    import pylab
    mra = MRA()
    N = 200
    x = numpy.linspace(-2, 2, N)
    y = x ** 4 - 3 * x ** 2 + x
    z = y + scipy.random.randn(N) * 0.5
    data = DataContainer.FieldContainer(
        z, dimensions=[DataContainer.FieldContainer(x, unit='1m')]
        )
    mra.paramScale.value = "1.0m"
    roots = mra.mra(data)
    pylab.plot(x, y, x, z)
    pylab.vlines(roots.data, -4, 4)
    pylab.vlines(roots.data + roots.error, -4, 4, linestyle='dashed')
    pylab.vlines(roots.data - roots.error, -4, 4, linestyle='dashed')
    pylab.show()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""

import numpy
from pyphant.core import (Worker, Connectors, DataContainer)
import scipy.interpolate
from pyphant import quantities
import copy
import logging
import pkg_resources
_logger = logging.getLogger("pyphant")


def createModel(model):
    xData = model.data[u'\\Psi_{3}'].tolist()[0]
    yData = model.data[u'\\Psi_{1}'].tolist()[0]
    return scipy.interpolate.interp1d(
        xData,
        yData,
        kind='linear',
        bounds_error=False,
        fill_value=0.0
        )


class OscThickness2CurrentDensity(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Convert Thickness to Current Density"
    _sockets = [
        ("osc", Connectors.TYPE_IMAGE),
        ("model", Connectors.TYPE_ARRAY)
        ]
    _params = [
        (
            "diameter",
            u"Diameter of probing area (eg. 800mum)",
            'unknown',
            None
            )
        ]

    @Worker.plug(Connectors.TYPE_IMAGE)
    def calcCurrentDensity(self, osc, model, subscriber=0):
        #Compute conversion factor between units
        #{u'\\Psi_{1}': Quantity(1.0,'mA/cm**2'),
        # u'\\Psi_{2}': 1.0, u'\\Psi_{3}': Quantity(1.0,'nm'),
        # u'\\Psi_{4}': 1.0, u'\\Psi_{0}': 1.0}
        unit = model._units[u'\\Psi_{3}']
        inputUnit = osc.unit
        msg = 'Units of input and output data are not compatible.'
        assert inputUnit.isCompatible(unit.unit), msg
        factor = inputUnit.inUnitsOf(unit.unit).value
        #Get model
        modelOperator = createModel(model)
        #Apply model
        inputData = factor * osc.data
        resultData = modelOperator(inputData)
        if resultData.min() == 0 and resultData.max() == 0:
            _logger.warning("The model does not describe the considered data.")
        #Incorporate area of probe
        if self.paramDiameter.value == 'unknown':
            factor = 1.0
        else:
            diameter = quantities.Quantity(
                self.paramDiameter.value.encode('latin-1')
                )
            factor = 0.25 * numpy.pi * diameter ** 2
        scaledUnit = factor * model._units[u'\\Psi_{1}']
        #Build Fieldcontainer
        result = DataContainer.FieldContainer(
            resultData,
            mask=osc.mask,
            longname=u'maximal short current density',
            shortname=u'\sigma',
            unit=scaledUnit,
            rescale=True,
            dimensions=copy.deepcopy(osc.dimensions)
            )
        result.seal()
        return result



# -*- coding: utf-8 -*-

# Copyright (c) 2007-2009, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""

import numpy
from pyphant.core import (Worker, Connectors)
from pyphant.quantities import Quantity
import logging
import copy
import pkg_resources


class OscThicknessCorrector(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Correct Thickness"
    _sockets = [("osc", Connectors.TYPE_ARRAY)]
    _params = [
        ("method", "Correct thickness for",
         ["Spin Coating", "Printing"], None),
        ("max_correction", "Max correction", "30 nm", None)
        ]

    def inithook(self):
        self._logger = logging.getLogger("pyphant")

    def perform_spincoat_correction(self, x, y, uncorrected_t):
        t = copy.deepcopy(uncorrected_t)
        r = numpy.sqrt(x.data ** 2 + y.data ** 2)
        r_min = r.min()
        r_max = r.max()
        correction = Quantity(self.paramMax_correction.value) / t.unit
        t.data = t.data + correction * ((r - r_min) / (r_max - r_min))
        t.longname = 'thickness corrected for spin coating'
        t.shortname = 't_c'
        return t

    def perform_print_correction(self, x, raw_y, uncorrected_t):
        t = copy.deepcopy(uncorrected_t)
        y = raw_y.data
        d = 1.9 * y ** 2 + 19.3 * y + 49
        t.data = t.data - d
        t.longname = 'thickness corrected for printing'
        t.shortname = 't_c'
        return t

    @Worker.plug(Connectors.TYPE_IMAGE)
    def correct(self, osc, subscriber=0):
        x = osc[u'x-position']
        y = osc[u'y-position']
        t = osc[u'thickness']
        method = self.paramMethod.value
        if  method == "Spin Coating":
            corrected_t = self.perform_spincoat_correction(x, y, t)
        elif method == "Printing":
            corrected_t = self.perform_print_correction(x, y, t)
        else:
            raise RuntimeError("Unknown correction method %s." % (method, ))
        corrected_t.seal()
        return corrected_t



# -*- coding: utf-8 -*-

# Copyright (c) 2008-2009, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
The Compare Fields Worker is a class of Pyphant's OSC Toolbox.
"""

import numpy
from pyphant.core import (Worker, Connectors, DataContainer)
import pkg_resources


class CompareFields(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Compare"
    _sockets = [("field1", Connectors.TYPE_IMAGE),
                ("field2", Connectors.TYPE_IMAGE)]
    _params = []

    @Worker.plug(Connectors.TYPE_IMAGE)
    def extract(self, field1, field2, subscriber=0):
        result = DataContainer.FieldContainer(numpy.array([field1, field2]))
        result.seal()
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2008, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""usage = "usage: %prog [options] archivename"

Converts ZIP or TAR archives to the Full Metadata File (FMF) format. The input archive has to contain either absorption measurements (Riede-INI) or simulated absorption curves (KSH-INI).

Example file header for KSH-INI:

[GLOBAL]
AUTHOR=Kristian O. Sylvester-Hvid
DATE=2007-12-07
TITLE=Simulated abs spectrum for P3HT:PCBM solar cell - 40nm blend layer
[SPALTENBESCHRIFTUNG]
SPALTE1=Wavelength[nm]
SPALTE2=Absorption[]
[MESSDATEN]
300 0.9399939
302 0.94043474


Example file header for Riede-INI:

[GLOBAL]
SUBSTRAT_TYP=T_norm-Space-resolved
SUBSTRAT_NAME=T00039
PIXEL=9
DATUM=26.10.2007
UHRZEIT=16:51:09
OPERATOR=KSH
HALTER=Space_resolved_T_norm
SUBSTRAT_POSITION=2
TISCH_POS_X=74.3000
TISCH_POS_Y=45.0000
FILTERRAD=<ohne>
KOMMENTAR="Absorption measurement of annealed (150C, 15min) T39 with inverted Al reference"
BELEUCHTUNG=0.0000E+0
MESSMODUL=ABS
RELAIS=<keine>
KOMMENTAR_PIXEL=" "
BATCHNUMBER=3276253246
LAMPE=<keine>
REFZELLE=<keine>
REF_CURRENT=0.0000E+0
[MESSUNG]
INTEGRATION_TIME=50
SCANS=10
CORRECT_DYNAMIC_DARK=1
FORGET_PERCENTAGE=100
SATURATION_DETECTION=0
SMOOTH_PIXEL=1
[SPALTENBESCHRIFTUNG]
SPALTE1=Wellenlänge[nm]
SPALTE2=ScopRaw[counts]
SPALTE3=Unkorrigierte Reflexion[counts]
SPALTE4=DarkRef[counts]
SPALTE5=WhiteRef[counts]
[MESSDATEN]
276.1833E+0	257.1975E+0	3.4475E+0	309.2355E+0	294.1412E+0
276.7759E+0	98.5742E+0	-295.3656E-3	114.5368E+0	168.5804E+0
"""
import tarfile,zipfile
import re
import datetime
import StringIO
from optparse import OptionParser
import platform,os
import pyphant.core.Helpers

parser= OptionParser(__doc__)
parser.add_option("-i", "--input-format", dest="iniFormat",default='RiedeINI',
                  help="FORMAT of input data (default is RiedeINI)", metavar="FORMAT",
                  choices=('RiedeINI','KSH-INI'))
(options, args) = parser.parse_args()
archiveName = args[0]
iniFormat = options.iniFormat
print "Converting %s from %s format to Full Metadata File format." % (archiveName,iniFormat)


class archive:
    def __init__(self, archiveName):
        if archiveName.endswith('tar'):
            self.type = 'TAR'
            self.archiveIN = tarfile.TarFile(archiveName)
        elif archiveName.endswith('zip'):
            self.type = 'ZIP'
            self.archiveIN = zipfile.ZipFile(archiveName)

    def names(self):
        if self.type == 'TAR':
            return [item.name for item in self.archiveIN]
        elif self.type == 'ZIP':
            return self.archiveIN.namelist()

    def extractedfile(self,filename):
        if self.type == 'TAR':
            datfile = self.archiveIN.extractfile(filename)
            return datfile.readlines()
        elif self.type == 'ZIP':
            datfile = self.archiveIN.read(filename)
            return StringIO.StringIO(datfile)

    def close(self):
        self.archiveIN.close()

searcher = re.compile("Sim_abs_(\d+)(\w+).dat")
archiveIN = archive(archiveName)

zip = zipfile.ZipFile(archiveName[:-4]+'_patched.zip','w')

def annotation():
    stream = ''
    stream += 'patched date: %s\n' % modDate
    stream += 'patched by: %s\n' % pyphant.core.Helpers.getUsername()
    stream += 'place: Freiburg i. Brsg.\n'
    stream += 'organization: Freiburger Materialforschungszentrum (FMF)\n'
    return stream

for datInfo in archiveIN.names():
    modDate = datetime.datetime.now()
    if iniFormat == 'RiedeINI':
        stream = ''
    elif iniFormat == 'KSH-INI':
        stream = "; -*- coding: utf-8 -*-\n"
#    datFile = archiveIN.extractfile(datInfo)
    width = searcher.match(datInfo)
    modus = None
    x = None
    y = None
    for line in archiveIN.extractedfile(datInfo):
        if line.startswith('[GLOBAL]'):
            line = '[*reference]\n'
        elif line.startswith('[SPALTENBESCHRIFTUNG]'):
            line = '[*data definitions]\n'
        elif line.startswith('[MESSDATEN]'):
            modus = '*data'
            line = '[*data]\n'
        elif line.startswith('SPALTE1'):
            line = 'wavelength: \\lambda [nm]\n'
        if iniFormat == 'RiedeINI':
            if line.startswith('TITLE'):
                stream += annotation()
                stream += "WIDTH=%s %s\n" % width.groups()
            elif line.startswith('SPALTE2'):
                line = 'scope raw intensity: I(\\lambda)\n'
            elif line.startswith('SPALTE3'):
                line = 'uncorrected reflexion: R(\\lambda)\n'
            elif line.startswith('SPALTE4'):
                line = 'dark reference intensity: I_d(\\lambda)\n'
            elif line.startswith('SPALTE5'):
                line = 'white reference intensity: I_0(\\lambda)\n'
            elif not line.startswith('['):
                words = line.split('=')
                if len(words)>1:
                    line = "%s: %s" % (words[0].lower().replace(' ','-'),words[1])
            if line.startswith('tisch_pos_x'):
                x = line.split(':')[1][:-2]
                line = 'horizontal_table_position: x = %s mm\n' % x
            if line.startswith('tisch_pos_y'):
                y = line.split(':')[1][:-2]
                line = 'vertical_table_position: y = %s mm\n' % y
            if line.startswith('kommentar:'):
                stream += annotation()
                line = 'title: %s at (x,y)=(%s,%s)\"\n' % (line.split(':')[1][1:-3],x,y)
        elif iniFormat == 'KSH-INI':
            if line.startswith('AUTHOR') or line.startswith('OPERATOR'):
                line = 'author: %s\n' % line.split('=')[1][:-2]
            elif line.startswith('TITLE'):
                stream += annotation()
                stream += "thickness: h = %s %s\n" % width.groups()
            elif line.startswith('SPALTE2'):
                line = 'absorption: A(\\lambda)\n'
        line = line.replace('<','').replace('>','').replace('\x0d','')
        line = line.replace('\" \"','ohne')
        stream += line
    info=zipfile.ZipInfo(datInfo,date_time=modDate.utctimetuple()[:6])
    info.compress_type = zipfile.ZIP_DEFLATED
    info.external_attr = 2175008768L
    if iniFormat == 'RiedeINI':
        zip.writestr(info,stream.encode('cp1252'))
    elif iniFormat == 'KSH-INI':
        zip.writestr(info,stream.encode('utf-8'))
archiveIN.close()
zip.close()



# -*- coding: utf-8 -*-
# Copyright (c) 2007-2008, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
The Estimate Error worker is a class of Pyphant's OSC-Toolbox. It
evaluates the error caused by noise for every pixel in a field.
"""

import numpy
from pyphant.core import (Worker, Connectors)
import logging
import copy
import pkg_resources

_logger = logging.getLogger("pyphant")


def localNoise(y, samples=50):
    length = len(y)
    samples2 = samples / 2
    sampleM1 = samples - 1
    sampleMod = samples % 2
    #Initialize extended y-vector
    yExtended = numpy.zeros(length + samples - samples % 2, 'float')
    #Mirroring to the left
    yExtended[:samples2] = y[samples2:0:-1]
    #Copy of array
    yExtended[samples2:length + samples2] = y
    #Mirroring to the right
    yExtended[length + samples2:] = y[-2:-2 - samples2:-1]
    #Inititalize error vector
    error = numpy.zeros(y.shape, 'float')
    #Compute experimental standard deviation
    for i in xrange(samples2, length + samples2):
        sample = yExtended[i - samples2:i + samples2 + sampleMod]
        yMean = numpy.mean(sample)
        error[i - samples2] = numpy.sqrt(numpy.sum((yMean - sample) ** 2) /
                                         sampleM1)
    return error


class ErrorEstimator(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Estimate Error"
    _sockets = [("osc", Connectors.TYPE_IMAGE)]
    _params = [
        ("dA", u"Window Size in number of data points", 10, None)
        ]

    @Worker.plug(Connectors.TYPE_IMAGE)
    def estimate(self, osc, subscriber=0):
        xCon = osc.data
        noise = numpy.zeros(xCon.shape, 'float')
        intervallLength = self.paramDA.value
        count = xCon.shape[0]
        #Loop over all rows
        for i in xrange(count):
            x = xCon[i]
            noise[i] = localNoise(x, self.paramDA.value)
            subscriber %= float(i + 1) / count * 100.0
        result = copy.deepcopy(osc)
        result.error = noise
        result.seal()
        return result



# -*- coding: utf-8 -*-
#!Pyphant's ExtremumFinder worker
#!-------------------------------
# Copyright (c) 2007-2008, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
Pyphant module computing the local extrema of one-dimensional
sampled fields. If a two-dimensional field is provided as input, the
algorithm loops over the 0th dimension denoting the y-axis, which
corresponds to an iteration over the rows of the data matrix.
"""

import numpy
from pyphant.core import (Worker, Connectors, DataContainer)
import pkg_resources


class ExtremumFinder(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant.osc").version
    name = "Find Extremum"
    _sockets = [("field", Connectors.TYPE_IMAGE)]
    _params = [
        ("extremum", u"extremum", [u"minima", u"maxima", u"both"], None)
        ]

    @Worker.plug(Connectors.TYPE_IMAGE)
    def locate(self, field, subscriber=0):
        #Determine the number of rows $N_{rows}$
        #for which the local extrema have to be found.
        if len(field.dimensions) == 1:
            Nrows = 1
        else:
            Nrows = len(field.dimensions[0].data)
        #Find local extrema $\vec{x}_0$
        x0, extremaCurv, extremaError, extremaPos = \
            findLocalExtrema(field, Nrows)
        #Map roots and curvatures to arrays
        if Nrows == 1:
            X0 = numpy.array(x0)
            XCurv = numpy.array(extremaCurv[0])
            XError = numpy.array(extremaError[0])
        else:
            maxLen = max(map(len, extremaPos))
            X0 = numpy.zeros((Nrows, maxLen), 'f')
            X0[:] = numpy.NaN
            XCurv = X0.copy()
            XError = X0.copy()
            for i in xrange(Nrows):
                numExt = len(extremaPos[i])
                if numExt == 1:
                    X0[i, 0] = extremaPos[i][0]
                    XCurv[i, 0] = extremaCurv[i]
                    XError[i, 0] = extremaError[i]
                else:
                    for j in xrange(numExt):
                        X0[i, j] = extremaPos[i][j]
                        XCurv[i, j] = extremaCurv[i][j]
                        XError[i, j] = extremaError[i][j]
        extremaType = self.paramExtremum.value
        if extremaType == u'minima':
            X0 = numpy.where(XCurv > 0, X0, numpy.nan)
            error = numpy.where(XCurv > 0, XError, numpy.nan)
        elif extremaType == u'maxima':
            X0 = numpy.where(XCurv < 0, X0, numpy.nan)
            error = numpy.where(XCurv < 0, XError, numpy.nan)
        else:
            extremaType = u'extrema'
            error = XError
        xName = field.dimensions[-1].longname
        xSym = field.dimensions[-1].shortname
        yName = field.longname
        ySym = field.shortname
        if numpy.sometrue(numpy.isnan(X0)):
            if len(field.dimensions) == 1:
                roots = DataContainer.FieldContainer(
                    numpy.extract(numpy.logical_not(numpy.isnan(X0)), X0),
                    unit=field.dimensions[-1].unit,
                    longname="%s of the local %s of %s" % (
                        xName, extremaType, yName
                        ),
                    shortname="%s_0" % xSym
                    )
            else:
                roots = DataContainer.FieldContainer(
                    X0.transpose(),
                    mask=numpy.isnan(X0).transpose(),
                    unit=field.dimensions[-1].unit,
                    longname="%s of the local %s of %s" % (
                        xName, extremaType, yName
                        ),
                    shortname="%s_0" % xSym
                    )
        else:
            roots = DataContainer.FieldContainer(
                X0.transpose(),
                unit=field.dimensions[-1].unit,
                longname="%s of the local %s of %s" % (
                    xName, extremaType, yName
                    ),
                shortname="%s_0" % xSym
                )
        if field.error != None:
            if len(field.dimensions) == 1:
                roots.error = numpy.extract(
                    numpy.logical_not(numpy.isnan(X0)), error
                    )
            else:
                roots.error = error.transpose()
        else:
            roots.error = None
        if len(field.data.shape) == 2:
            roots.dimensions[-1] = field.dimensions[0]
        roots.seal()
        return roots


def findLocalExtrema(field, Nrows):
    #Init nested lists ll_x0, ll_sigmaX0 and ll_curv
    #which are going to hold one list per analysed data row.
    ll_x0 = []       # Nested list for $\vec{x}_{0,i}$ with $i=0,N_{rows}-1$
    ll_sigmaX0 = []  # Nested list for $\sigma_{x_{0,i}}$ with $i=0,N_{rows}-1$
    ll_curv = []     # Nested list indicating local maximum (-1)
                     # or local minimum (1)
    #Because we are looking for local extrema of one-dimensional sampled
    #fields the last dimension is the sampled abscissa $\vec{x}$.
    x = field.dimensions[-1].data
    #Loop over all rows $N_{rows}$.
    for i in xrange(Nrows):
        #If a $1\times N_{rows}$ matrix is supplied,
        #save this row to vector $\vec{y}$.
        #Otherwise set vector $\vec{y}$ to the i$^\text{th}$ row
        #of matrix field.data and handle vector of errors $\vec{\sigma}_y$
        #accordingly. It is None, if no error is given.
        sigmaY = field.error
        if Nrows == 1:
            y = field.data
        else:
            y = field.data[i]
            if field.error != None:
                sigmaY = field.error[i]
        x0, l_sigmaX0, dyy = findLocalExtrema1D(y, x, sigmaY)
        ll_x0.append(numpy.array(x0))
        ll_sigmaX0.append(numpy.array(l_sigmaX0))
        ll_curv.append(numpy.array(dyy))
    return x0, ll_curv, ll_sigmaX0, ll_x0


def findLocalExtrema1D(y, x, sigmaY=None):
    #Compute differences $\vec{\Delta}_y$ of data vector $\vec{y}$.
    #The differencing reduces the dimensionalty of the vector by one:
    #$\text{dim}\vec{\Delta}_y=\text{dim}\vec{x}-1$.
    DeltaY   = numpy.diff(y)
    #Test if the sign of successive elements of DeltaY change sign.
    #These elements are candidates for the estimation of local extrema.
    #The result is a vector b_x0 of booleans with
    #$\text{dim}\vec{x}_{0,\text{b}}=\text{dim}\vec{x}-2$.
    #From b_x0[j]==True follows $x_{0,j}\in[x_{j+1},x{j+2}]$.
    #Note, that b_x0[j]==b_x0[j+1]=True indicate a special case,
    #which maps to one local extremum $x_{0,j}\in[x_{j+1},x_{j+2}]$.
    b_x0 = numpy.sign(DeltaY[:-1]) != numpy.sign(DeltaY[1:])
    #Init list l_x0 for collecting the local extrema.
    l_x0 = []
    #Init list l_sigmaX0 for collecting the estimation errors
    #of locale extrema positions $\vec{x}_0$.
    l_sigmaX0 = []
    #Init list l_curv_sign for collecting the sign of the curvature
    #at the position of the locale extrema.
    l_curv_sign   = []
    #If one or more local extrema have been found, estimate its position,
    #otherwise set its position to NaN.
    if numpy.sometrue(b_x0):
        #Remove successive True values, which occur do to symmetrically
        #boxed or exact local extrema.
        b_x0[1:] = numpy.where(
            numpy.logical_and(b_x0[:-1], b_x0[1:]), False, b_x0[1:]
            )
        #Compute vector index referencing the True elements of b_x0.
        index = numpy.extract(b_x0, numpy.arange(len(DeltaY) - 1))
        skipOne = False
        for j in index:
            if skipOne:
                skipOne = False
            else:
                if sigmaY == None:
                    x0, sigmaX0, curv_sign = estimateExtremumPosition(
                        y[j:j + 3], x[j:j + 3]
                        )
                else:
                    x0, sigmaX0, curv_sign = estimateExtremumPosition(
                        y[j:j + 3], x[j:j + 3], sigmaY=sigmaY[j:j + 3]
                        )
                l_x0.append(x0)
                l_sigmaX0.append(sigmaX0)
                l_curv_sign.append(curv_sign)
    else:  # No local extremum found.
        l_x0.append(numpy.NaN)
        l_sigmaX0.append(numpy.NaN)
        l_curv_sign.append(numpy.NaN)
    return l_x0, l_sigmaX0, l_curv_sign


#$ \section{Function \lstinline!estimateExtremumPosition(y,x,sigmaY=None)!}
#$ Estimate the extremum position from three sample points $(x_0,y_0)$,
#$ $(x_1,y_1)$, and $(x_1,y_1)$ by a linear model. The sample points are provided as
#$ vectors $\vec{x}=(x_0,x_1,x_2)$ and $\vec{y}=(y_0,y_1,y_2)$. The middle sample
#$ point $(x_1,y1)$ separates two bins. For each bin the slope $y^\prime$ is calculated
#$ as finite difference. These slopes are assumed to be located at the centre of each bin, such that
#$ the slopes $\left((x_0+x_1)/2,(y_1-y_0)/(x_1-x_0)\right)$ and
#$ $\left((x_1+x_2)/2,(y_2-y_1)/(x_2-x_1)\right)$ can be compiled to a linear equation, whose root is
#$ an estimate for the position of the local extremum:
#$ \begin{equation}\label{Eq:estimator}
#$ \tilde{x}_0=\frac{1}{2}(x_0+x_1)-\frac{\frac{1}{2}(x_2-x_0)}{\frac{y_2-y_1}{x_2-x_1}-\frac{y_1-y_0}{x_1-x_0}}\frac{y_1-y_0}{x_1-x_0}
#$ \end{equation} and its error
#$ \begin{equation}\label{Eq:uncertainty}
#$ \sigma_{\tilde{x}_0}=R\cdot(\sigma_{y,0}|y_2-y_1|+\sigma_{y,1}|y_2-y_0|+\sigma_{y,3}|y_1-y_0|)
#$ \end{equation} with
#$ \begin{equation*}
#$ R=\frac{1}{2}\left|\frac{(x_1-x_0)(x_2-x_0)(x_2-x_1)}{[y_0(x_2-x_1)+y_1(x_0-x_2)+y_2(x_1-x_0)]^2}\right|.
#$ \end{equation*}
def estimateExtremumPosition(y, x, sigmaY=None):
    """
    Estimate the extremum position

    Estimate the extremeum position from three sample points,
    whose x- and y-coordinates are given as numpy arrays x and y.
    The middle sample point separates two bins.
    For each bin a slope is calculated as finite difference.
    Both slopes are assumed to be located at the centre of each bin,
    which leads to a linear equation for the estimation
    of the position of the local extremum.
    If an y-error is specified an estimation error is computed
    from error propagation.
    """
    #Compute the width of left and right bin. The bin width has to be finite.
    deltaXleft = x[1] - x[0]
    deltaXright = x[2] - x[1]
    if deltaXleft == 0 or deltaXright == 0:
        raise ValueError("Both bins need to have a finite width.")
    #Compute the centres of left and right bin.
    #The centre should not be identical.
    xCleft = 0.5 * (x[0] + x[1])
    xCright = 0.5 * (x[1] + x[2])
    if xCleft == xCright:
        raise ValueError(
            "The centres of the left and the right bin cannot be identical."
            )
    #Compute the difference of the sampled values.
    deltaYleft = y[1] - y[0]
    deltaYright = y[2] - y[1]
    #If the difference is zero in both bins, a constant region
    #has been detected and the algorithm should return NaN.
    #If the difference of the right bin is greater or lower than zero
    #a local minimum or local maximum has been detected, respectively.
    if deltaYleft == 0.0:
        if deltaYright == 0.0:  # constant region
            return numpy.NaN, numpy.NaN, numpy.NaN
        elif deltaYright > 0:   # local minimum
            curv_sign = 1.0
        else:                   # local maximum
            curv_sign = -1.0
    else:
        curv_sign = -numpy.sign(deltaYleft)
    # Estimate position of local extrema according to
    # Eq. $\text{(\ref{Eq:estimator})}$.
    x0 = xCleft - (xCright - xCleft) / (
        deltaYright / deltaXright - deltaYleft / deltaXleft
        ) * deltaYleft / deltaXleft
    # If an y-error has been provided, compute the estimation error
    # according to Eq. $\text{(\ref{Eq:uncertainty})}$.
    if sigmaY != None:
        numerator = 0.5 * deltaXleft * deltaXright * (x[2] - x[0])
        R = numerator / (y[0] * deltaXright + y[1] * (
                x[0] - x[2]) + y[2] * deltaXleft
                         ) ** 2
        partError = numpy.array([-deltaYright, y[2] - y[0], -deltaYleft])
        sigmaX0 = R * numpy.dot(sigmaY, numpy.abs(partError))
    else:
        sigmaX0 = numpy.NaN
    return x0, sigmaX0, curv_sign



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Pyphant Organic Solar Cells toolbox
Visit http://pyphant.sourceforge.net for more information.
"""


VERSION = '1.0b3'

from setuptools import setup, find_packages
from setuptools.command.test import test


class EnsureAggUnittests(test):
    def run_tests(self):
        from pyphant.mplbackend import ensure_mpl_backend
        ensure_mpl_backend('agg')
        return test.run_tests(self)


setup(
    name="pyphant.osc",
    version=VERSION,
    author="Andreas W. Liehr, Klaus Zimmermann",
    author_email='klaus.zimmermann@fmf.uni-freiburg.de',
    maintainer='Klaus Zimmermann',
    maintainer_email='klaus.zimmermann@fmf.uni-freiburg.de',
    license="BSD",
    description=__doc__,
    install_requires=[
        'pyphant>=1.0b3',
        'numpy',
        'scipy',
        'matplotlib',
        ],
    packages=find_packages(),
    entry_points="""
    [pyphant.workers]
    myeentry = OSC
    """,
    include_package_data=True,
    cmdclass={'test': EnsureAggUnittests},
    test_suite='OSC.tests'
    )



# -*- coding: utf-8 -*-

# Copyright (c) 2007-2008, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""Provides unittest classes TestExtremumFinder and TestExtremumFinderDiscontinuousDiscretisation.
"""



import sys
import unittest
import numpy, scipy, scipy.optimize
import OSC.ExtremumFinder as EF
from pyphant.core import DataContainer as DC
from pyphant.tests.Helpers import fixedPoints


class TestExtremumFinder(unittest.TestCase):
    """Sets up a mirror symmetric bistable potential with a continuous
    distretisation and computes its local extrema and the respective
    curvatures."""
    def setUp(self):
        self.n = 1000
        self.u = numpy.linspace(-1.5,1.5,self.n)
        self.LAMBDA = 0.5
        self.kappa1=0.0
        self.xField = DC.FieldContainer(self.u,
                                   unit = '1 m',
                                   longname = 'position',
                                   shortname = 'x')
        self.V = DC.FieldContainer(-self.LAMBDA/2* self.u**2 + self.u**4/4-self.u*self.kappa1,
                                   unit='1 V',dimensions=[self.xField],
                                   longname = 'electric potential',
                                   shortname=r'\varphi')

    def testRoots(self):
        """Test the correct computation of all local extrema for a bistable potential."""
        #Predict result
        x0,curv,mask = fixedPoints(numpy.array([self.LAMBDA]),kappa1=self.kappa1)
        curv[0]=numpy.NaN
        expectedResult = DC.FieldContainer(x0[0],
                                           unit = self.xField.unit,
                                           longname = 'position of the local extrema of electric potential',
                                           shortname = 'x_0')
        #Retrieve result from worker
        w = EF.ExtremumFinder(None)
        w.paramExtremum.value=u'both'
        result = w.locate(self.V)
        #Testing
        DC.assertEqual(result,expectedResult)

    def testMinima(self):
        """Test the correct computation of all local minima for a bistable potential."""
        #Predict result
        x0,curv,mask = fixedPoints(numpy.array([self.LAMBDA]),kappa1=self.kappa1)
        expectedResult = DC.FieldContainer(numpy.extract(curv[0]>0,x0[0]),
                                           unit = self.xField.unit,
                                           longname = 'position of the local minima of electric potential',
                                           shortname = 'x_0')
        #Retrieve result from worker
        w = EF.ExtremumFinder(None)
        w.paramExtremum.value=u'minima'
        result = w.locate(self.V)
        #Testing
        DC.assertEqual(result,expectedResult)

    def testMaxima(self):
        """Test the correct computation of all local maxima for a bistable potential."""
        #Predict result
        x0,curv,mask = fixedPoints(numpy.array([self.LAMBDA]),kappa1=self.kappa1)
        curv[0]=numpy.NaN
        expectedResult = DC.FieldContainer(numpy.extract(curv[0]<0,x0[0]),
                                           unit = self.xField.unit,
                                           longname = 'position of the local maxima of electric potential',
                                           shortname = 'x_0')
        #Retrieve result from worker
        w = EF.ExtremumFinder(None)
        w.paramExtremum.value=u'maxima'
        result = w.locate(self.V)
        #Testing
        DC.assertEqual(result,expectedResult)

class TestExtremumFinderTable(unittest.TestCase):
    """Sets up a mirror symmetric bistable potential with a continuous distretisation and computes its local extrema and the respective curvatures."""
    def setUp(self):
        self.n = 10000
        self.m = 10
        self.kappa1=0.0
        self.errLevelPos = 6
        self.errLevelCurv= 5
        self.test = DC.assertEqual

    def prepareDimensions(self):
        X,LAMB = scipy.meshgrid(numpy.linspace(-1.5,1.5,self.n),
                                numpy.linspace(-1.0,1.0,self.m))
        self.lambDim = LAMB[:,0]
        self.xDim = X

    def testRoots(self):
        """Test the correct computation of all local extrema for a bistable potential."""
        #Prepare dimensions
        self.prepareDimensions()
        lambField = DC.FieldContainer(self.lambDim,
                                      unit = '1 V / m**3',
                                      longname='parameter',
                                      shortname='\lambda')
        xField = DC.FieldContainer(self.xDim[0],
                                   unit = '1 m',
                                   longname = 'position',
                                   shortname = 'x')
        #Prepare potential
        V = []
        for i in xrange(len(lambField.data)):
            u = xField.data
            V.append(-lambField.data[i]/2* u**2 + u**4/4-u*self.kappa1)
        self.V = DC.FieldContainer(numpy.array(V),unit='1 V',dimensions=[lambField,xField],
                                   longname = 'electric potential',
                                   shortname=r'\varphi')
        #Predict result
        x0,curv,mask = fixedPoints(lambField.data,kappa1=self.kappa1)
        error = 1.0/curv.transpose()
        error[:] =numpy.nan
        data = x0.transpose()
        expectedResult = DC.FieldContainer(data,
                                           unit = xField.unit,
                                           mask = numpy.isnan(data),
                                           dimensions=[DC.generateIndex(0,3),lambField],
                                           longname = 'position of the local extrema of electric potential',
                                           shortname = 'x_0')
        #Configure worker
        w = EF.ExtremumFinder(None)
        w.paramExtremum.value=u'both'
        #Retrieve result from worker
        result = w.locate(self.V)
        self.test(result,expectedResult)

class TestExtremumFinderTableFail(TestExtremumFinderTable):
    """Sets up a mirror symmetric bistable potential with a continuous
    discretisation, computes its local extrema and the respective
    curvatures. This test succeeds, if the deviation between expected
    and computed result due to the insufficient spatial resolution is
    detected."""
    def setUp(self):
        self.n = 1000
        self.m = 10
        self.kappa1=0.0
        self.errLevelPos = 6
        self.errLevelCurv= 5
        self.test = self.assertNotEqual

class TestError(unittest.TestCase):
    def testParabelExactMinimum(self):
        x = numpy.array([-2,-1,0,1,2],'float')
        y = x**2
        inputField = DC.FieldContainer(y,
                                       error=numpy.repeat(0.1,len(y)),
                                       dimensions=[DC.FieldContainer(x,longname='abscissae',shortname='x')],
                                       longname='parabel',
                                       shortname='f'
                                       )
        expectedResult = DC.FieldContainer(numpy.array([0.0]),
                                           longname = 'position of the local minimum of parabel',
                                           shortname = 'x_0',
                                           error = numpy.array([0.05])
                                           )
        w = EF.ExtremumFinder(None)
        w.paramExtremum.value=u'minima'
        #Retrieve result from worker
        result = w.locate(inputField)
        DC.assertEqual(result,expectedResult)

    def testParabelSymmetricallyBoxedMinimum(self):
        x = numpy.array([-2,-1,0,1,2],'float')-0.5
        y = x**2
        inputField = DC.FieldContainer(y,
                                       error=numpy.repeat(0.1,len(y)),
                                       dimensions=[DC.FieldContainer(x,longname='abscissae',shortname='x')],
                                       longname='parabel',
                                       shortname='f'
                                       )
        expectedResult = DC.FieldContainer(numpy.array([0.0]),
                                           longname = 'position of the local minimum of parabel',
                                           shortname = 'x_0',
                                           error = numpy.array([0.1])
                                           )
        w = EF.ExtremumFinder(None)
        w.paramExtremum.value=u'minima'
        #Retrieve result from worker
        result = w.locate(inputField)
        DC.assertEqual(result,expectedResult)

    def testParabel(self):
        x = numpy.array([-2,-1,0,1,2],'float')+0.1
        y = x**2
        inputField = DC.FieldContainer(y,
                                       error=numpy.repeat(0.1,len(y)),
                                       dimensions=[DC.FieldContainer(x,longname='abscissae',shortname='x')],
                                       longname='parabel',
                                       shortname='f')
        error  = inputField.error[slice(0,1)] / (y[1]-2*y[2]+y[3])**2
        error *= numpy.abs(y[2]-y[3]) + numpy.abs(y[1]-y[3]) + numpy.abs(y[1]-y[2])
        expectedResult = DC.FieldContainer(numpy.array([0.0]),
                                           longname = 'position of the local minimum of parabel',
                                           shortname = 'x_0',
                                           error = error)
        w = EF.ExtremumFinder(None)
        w.paramExtremum.value=u'minima'
        #Retrieve result from worker
        result = w.locate(inputField)
        DC.assertEqual(result,expectedResult)

    def testParabelArray(self):
        x = numpy.array([-2,-1,0,1,2],'float')
        field = numpy.array([(x-0.2)**2-0.5,(x+0.1)**2])
        inputField = DC.FieldContainer(field,
                                       error=numpy.resize(numpy.repeat(0.1,len(field)),field.shape),
                                       dimensions=[DC.FieldContainer(numpy.array([1,2]),
                                                                     longname='type',shortname=r'\theta'),
                                                   DC.FieldContainer(x,longname='position',shortname='x')],
                                       longname='parabel',
                                       shortname='f')
        def error(y):
            error  = inputField.error[0,0] / (y[1]-2*y[2]+y[3])**2
            error *= numpy.abs(y[2]-y[3]) + numpy.abs(y[1]-y[3]) + numpy.abs(y[1]-y[2])
            return error
        expectedResult = DC.FieldContainer(numpy.array([[0.2,-0.1]]),
                                           longname = 'position of the local minima of parabel',
                                           shortname = 'x_0',
                                           error = numpy.array([[error(field[0]),error(field[1])]]))
        expectedResult.dimensions[-1] = DC.FieldContainer(numpy.array([1,2]),
                                                          longname='type',
                                                          shortname=r'\theta')
        w = EF.ExtremumFinder(None)
        w.paramExtremum.value=u'minima'
        #Retrieve result from worker
        result = w.locate(inputField)
        DC.assertEqual(result,expectedResult)

class TestEstimateExtremumPosition(unittest.TestCase):
    def setUp(self):
        self.x = numpy.array([2,3,4],'f')
        self.y = numpy.array([2,1,1.5],'f')
        self.zeros = numpy.zeros((3,),'f')
        self.error = numpy.array([0.01,0.01,0.01],'f')

    def testExactMinimum(self):
        y = numpy.array([2,1,2],'f')
        result =  EF.estimateExtremumPosition(y,self.x)
        self.assertEqual(result,(3.0,numpy.NaN,1.0))
        result =  EF.estimateExtremumPosition(y,self.x,sigmaY=self.zeros)
        self.assertEqual(result,(3.0,0.0,1.0))
        result =  EF.estimateExtremumPosition(y,self.x,sigmaY=self.error)
        numpy.testing.assert_almost_equal(numpy.array(result),
                                          numpy.array((3.0,0.005,1.0)))

    def testExactMaximum(self):
        y = numpy.array([2,3,2],'f')
        result =  EF.estimateExtremumPosition(y,self.x)
        self.assertEqual(result,(3.0,numpy.NaN,-1.0))
        result =  EF.estimateExtremumPosition(y,self.x,sigmaY=self.zeros)
        self.assertEqual(result,(3.0,0.0,-1.0))
        result =  EF.estimateExtremumPosition(y,self.x,sigmaY=self.error)
        numpy.testing.assert_almost_equal(numpy.array(result),
                                          numpy.array((3.0,0.005,-1.0)))

    def testSymmetricMinimum(self):
        y = numpy.array([2,2,3],'f')
        result =  EF.estimateExtremumPosition(y,self.x)
        self.assertEqual(result,(2.5,numpy.NaN,1.0))
        result =  EF.estimateExtremumPosition(y,self.x,sigmaY=self.zeros)
        self.assertEqual(result,(2.5,0.0,1.0))
        result =  EF.estimateExtremumPosition(y,self.x,sigmaY=self.error)
        numpy.testing.assert_almost_equal(numpy.array(result),
                                          numpy.array((2.5,0.02,1.0)))

    def testSymmetricMaximum(self):
        y = numpy.array([2,2,1],'f')
        result =  EF.estimateExtremumPosition(y,self.x)
        self.assertEqual(result,(2.5,numpy.NaN,-1.0))
        result =  EF.estimateExtremumPosition(y,self.x,sigmaY=self.zeros)
        self.assertEqual(result,(2.5,0.0,-1.0))
        result =  EF.estimateExtremumPosition(y,self.x,sigmaY=self.error)
        numpy.testing.assert_almost_equal(numpy.array(result),
                                          numpy.array((2.5,0.02,-1.0)))

    def testExactGeneric(self):
        result =  EF.estimateExtremumPosition(self.y,self.x)
        self.assertEqual(result,(2.5+1/1.5,numpy.NaN,1.0))
        result =  EF.estimateExtremumPosition(self.y,self.x,sigmaY=self.zeros)
        self.assertEqual(result,(2.5+1/1.5,0.0,1.0))
        result =  EF.estimateExtremumPosition(self.y,self.x,sigmaY=self.error)
        numpy.testing.assert_almost_equal(numpy.array(result),
                                          numpy.array((2.5+1/1.5,0.02/1.5**2,1.0)))

    def testConstantRegion(self):
        """A constant region does not have any local extrema and NaN values should be returned."""
        y = numpy.array([2,2,2],'f')
        result =  EF.estimateExtremumPosition(y,self.x)
        self.assertEqual(result,(numpy.NaN,numpy.NaN,numpy.NaN))
        result =  EF.estimateExtremumPosition(y,self.x,sigmaY=self.zeros)
        self.assertEqual(result,(numpy.NaN,numpy.NaN,numpy.NaN))

    def testEqualBinCentres(self):
        """Erroneous sampling points leading to identical bin centres have to intercepted."""
        x = numpy.array([2,3,2],'f')
        self.assertRaises(ValueError,EF.estimateExtremumPosition,self.y,x)

    def testFiniteBinWidth(self):
        """Erroneous sampling points leading to zero bin width have to intercepted."""
        x = numpy.array([2,2,2],'f')
        self.assertRaises(ValueError,EF.estimateExtremumPosition,self.y,x)
        x = numpy.array([2,2,3],'f')
        self.assertRaises(ValueError,EF.estimateExtremumPosition,self.y,x)
        x = numpy.array([2,3,3],'f')
        self.assertRaises(ValueError,EF.estimateExtremumPosition,self.y,x)

if __name__ == '__main__':
    unittest.main()
